@@ -14,9 +14,27 @@ require 'mkmf'
 extension_name = 'atomic_reference'
 dir_config(extension_name)
 
-case CONFIG["arch"]
-when /mswin32|mingw/
-    $CFLAGS += " -march=native"
+have_header "libkern/OSAtomic.h"
+
+def compiler_is_gcc
+  if CONFIG["GCC"] && CONFIG["GCC"] != ""
+    return true
+  elsif ( # This could stand to be more generic...  but I am afraid.
+    CONFIG["CC"] =~ /\bgcc\b/
+  )
+    return true
+  end
+  return false
+end
+
+
+if compiler_is_gcc
+  case CONFIG["arch"]
+  when /mswin32|mingw|solaris/
+      $CFLAGS += " -march=native"
+  when 'i686-linux'
+      $CFLAGS += " -march=i686"
+  end
 end
 
 try_run(<<CODE,$CFLAGS) && ($defs << '-DHAVE_GCC_CAS')
